use std::net::UdpSocket;

#[derive(Default, Debug)]
pub struct GamepadRepresentation {
    // 1 byte
    action_a: bool,
    action_b: bool,
    action_x: bool,
    action_y: bool,
    arrow_up: bool,
    arrow_left: bool,
    arrow_right: bool,
    arrow_down: bool,
    // 2 byte
    bumper_left: bool,
    bumper_right: bool,
    menu_right: bool,
    menu: bool,
    menu_left: bool,
    cam: bool,
    joy: bool,
    // 3 bytes
    cam_x: i8,
    // 4 bytes
    cam_y: i8,
    // 5 bytes
    joy_x: i8,
    // 6 bytes
    joy_y: i8,
    // 7 bytes
    trigger_left: u8,
    // 8 bytes
    trigger_right: u8,
}

fn bools_from_u8(results: &mut [&mut bool], mut value: u8) {
    for result in results {
        **result = (value & 1) == 1;
        value >>= 1;
    }
}

impl From<&[u8; 8]> for GamepadRepresentation {
    fn from(bytes: &[u8; 8]) -> Self {
        let mut g = GamepadRepresentation::default();
        bools_from_u8(
            &mut [
                &mut g.action_a,
                &mut g.action_b,
                &mut g.action_x,
                &mut g.action_y,
                &mut g.arrow_down,
                &mut g.arrow_left,
                &mut g.arrow_right,
                &mut g.arrow_up,
            ],
            bytes[0],
        );
        bools_from_u8(
            &mut [
                &mut g.bumper_left,
                &mut g.bumper_right,
                &mut g.menu_left,
                &mut g.menu,
                &mut g.menu_right,
                &mut g.joy,
                &mut g.cam,
            ],
            bytes[1],
        );

        g.joy_x = i8::from_le_bytes([bytes[2]]);
        g.joy_y = i8::from_le_bytes([bytes[3]]);
        g.cam_x = i8::from_le_bytes([bytes[4]]);
        g.cam_y = i8::from_le_bytes([bytes[5]]);
        g.trigger_left = u8::from_le_bytes([bytes[6]]);
        g.trigger_right = u8::from_le_bytes([bytes[7]]);
        g
    }
}

fn main() {
    let listener = UdpSocket::bind("127.0.0.1:1069").unwrap();
    loop {
        let mut gamepad_bytes = [0; 8];
        let _ = listener.recv_from(&mut gamepad_bytes);
        let gamepad_representation = GamepadRepresentation::from(&gamepad_bytes);
        print!("{esc}[2J{esc}[1;1H", esc = 27 as char);
        println!("{:#?}", gamepad_representation);
    }
}
