mod gamepad;
use gamepad::GamepadRepresentation;
use std::{sync::Arc, time::Duration};
use stick::Listener;
use tokio::{net::TcpSocket, sync::Mutex};

const PACKET_SEND_RATE: Duration = Duration::from_millis(8);
const REMOTE_ADDR: &str = "192.168.106.232:1069";

#[derive(Default)]
struct ControllerClient {
    internal_gamepad: Arc<Mutex<GamepadRepresentation>>,
}

impl ControllerClient {
    async fn receive_input(&self) {
        let mut controller = Listener::default().await;
        println!("{}", controller.name());
        loop {
            let event = (&mut controller).await;
            self.internal_gamepad.lock().await.process_event(event);
        }
    }
    async fn send_input(&self) {
        let sock = TcpSocket::new_v4().unwrap();
        let mut tcp_stream = sock.connect(REMOTE_ADDR.parse().unwrap()).await.unwrap();
        println!("Connected to {}.", REMOTE_ADDR);
        loop {
            self.internal_gamepad
                .lock()
                .await
                .send(&mut tcp_stream)
                .await;
            tokio::time::sleep(PACKET_SEND_RATE).await;
        }
    }
}

#[tokio::main]
async fn main() {
    let state = ControllerClient::default();
    tokio::join!(state.receive_input(), state.send_input());
}
