#![no_std]
#![no_main]

use esp32s3_hal::{
    clock::ClockControl,
    gpio::IO,
    mcpwm::{operator::PwmPinConfig, timer::PwmWorkingMode, PeripheralClockConfig, MCPWM},
};
use machine_hal::{motor::Motor, prelude::*};

#[entry]
fn main() -> ! {
    let peripherals = Peripherals::take();
    let mut system = peripherals.SYSTEM.split();
    let clocks = ClockControl::boot_defaults(system.clock_control).freeze();

    let io = IO::new(peripherals.GPIO, peripherals.IO_MUX);

    let motor1_pwm = io.pins.gpio4.into_push_pull_output();
    let motor1_direction = io.pins.gpio5.into_push_pull_output();
    let motor2_pwm = io.pins.gpio6.into_push_pull_output();
    let motor2_direction = io.pins.gpio7.into_push_pull_output();
    let motor3_pwm = io.pins.gpio11.into_push_pull_output();
    let motor3_direction = io.pins.gpio12.into_push_pull_output();
    let motor4_pwm = io.pins.gpio13.into_push_pull_output();
    let motor4_direction = io.pins.gpio14.into_push_pull_output();

    // initialize peripheral
    let clock_cfg = PeripheralClockConfig::with_frequency(&clocks, 40u32.MHz()).unwrap();
    let mut mcpwm = MCPWM::new(
        peripherals.MCPWM0,
        clock_cfg,
        &mut system.peripheral_clock_control,
    );

    // connect operator0 to timer0
    mcpwm.operator0.set_timer(&mcpwm.timer0);
    mcpwm.operator1.set_timer(&mcpwm.timer1);

    // connect operator0 to pin
    let (motor1_pwm, motor2_pwm) = mcpwm.operator0.with_pins(
        motor1_pwm,
        PwmPinConfig::UP_ACTIVE_HIGH,
        motor2_pwm,
        PwmPinConfig::UP_ACTIVE_HIGH,
    );

    let (motor3_pwm, motor4_pwm) = mcpwm.operator1.with_pins(
        motor3_pwm,
        PwmPinConfig::UP_ACTIVE_HIGH,
        motor4_pwm,
        PwmPinConfig::UP_ACTIVE_HIGH,
    );
    // start timer with timestamp values in the range of 0..=99 and a frequency of
    // 20 kHz
    let timer_clock_cfg = clock_cfg
        .timer_clock_with_frequency(127, PwmWorkingMode::Increase, 20u32.kHz())
        .unwrap();
    mcpwm.timer0.start(timer_clock_cfg);
    mcpwm.timer1.start(timer_clock_cfg);

    let mut motor1 = Motor::new(motor1_pwm, motor1_direction);
    let mut motor2 = Motor::new(motor2_pwm, motor2_direction);
    let mut motor3 = Motor::new(motor3_pwm, motor3_direction);
    let mut motor4 = Motor::new(motor4_pwm, motor4_direction);
    motor1.set_speed(i8::MAX);
    motor2.set_speed(i8::MAX);
    motor3.set_speed(i8::MAX);
    motor4.set_speed(i8::MAX);
    loop {
        // motor1.set
    }
}
