use esp32s3_hal::mcpwm::{operator::PwmPin, PwmPeripheral};

pub trait SimpleMotor {
    fn set_speed(&mut self, speed: i8);
}

pub struct Motor<
    'd,
    PIN: esp32s3_hal::gpio::OutputPin,
    PWM: PwmPeripheral,
    const OP: u8,
    const IS_A: bool,
    OUTPUT: embedded_hal::digital::v2::OutputPin,
> {
    pwm: PwmPin<'d, PIN, PWM, OP, IS_A>,
    direction: OUTPUT,
}

impl<
        'd,
        PIN: esp32s3_hal::gpio::OutputPin,
        PWM: PwmPeripheral,
        const OP: u8,
        const IS_A: bool,
        OUTPUT: embedded_hal::digital::v2::OutputPin,
    > Motor<'d, PIN, PWM, OP, IS_A, OUTPUT>
{
    pub fn new(pwm: PwmPin<'d, PIN, PWM, OP, IS_A>, direction: OUTPUT) -> Self {
        Self { pwm, direction }
    }
    pub fn set_speed(&mut self, speed: i8) {
        self.pwm.set_timestamp(speed.abs() as u16);
        if speed.is_positive() {
            let _ = self.direction.set_low();
        } else {
            let _ = self.direction.set_high();
        }
    }
}

impl<
        'd,
        PIN: esp32s3_hal::gpio::OutputPin,
        PWM: PwmPeripheral,
        const OP: u8,
        const IS_A: bool,
        OUTPUT: embedded_hal::digital::v2::OutputPin,
    > SimpleMotor for Motor<'d, PIN, PWM, OP, IS_A, OUTPUT>
{
    fn set_speed(&mut self, speed: i8) {
        self.pwm.set_timestamp(speed.abs() as u16);
        if speed.is_positive() {
            let _ = self.direction.set_low();
        } else {
            let _ = self.direction.set_high();
        }
    }
}
